-- Non Functional Test Metrics Database Schema.

DROP TABLE IF EXISTS data;
DROP TABLE IF EXISTS test;
DROP TABLE IF EXISTS project;

CREATE SEQUENCE project_id_seq;
CREATE TABLE project (
    id      INTEGER PRIMARY KEY default nextval('project_id_seq'),
    name    varchar(128) NOT NULL,
    UNIQUE (name)    
);
ALTER SEQUENCE project_id_seq owned by project.id;

CREATE SEQUENCE test_id_seq;
CREATE TABLE test (
    id          INTEGER PRIMARY KEY default nextval('test_id_seq'),
    name        varchar(128) NOT NULL,
    project_id  integer,
    FOREIGN KEY (project_id) REFERENCES project(id),
    UNIQUE (project_id, name)
);
ALTER SEQUENCE test_id_seq owned by test.id;

CREATE SEQUENCE data_id_seq;
CREATE TABLE data (
    id              INTEGER PRIMARY KEY default nextval('data_id_seq'),
    date_entered    timestamp with time zone NOT NULL,
    data            json NOT NULL,
    test_id         integer,
    FOREIGN KEY (test_id) REFERENCES test(id)
);
ALTER SEQUENCE data_id_seq owned by data.id;
-- We will frequently be looking for data between two date ranges. Index the
-- date_entered column so this is efficient:
CREATE INDEX data_date_entered_index ON data(date_entered);

-- Tables for authentication below here.

-- The client table stores secret resource owner keys, as well as public
-- client and resource owner keys. We also store some information about
-- the client.
CREATE SEQUENCE client_id_seq;
CREATE TABLE client (
    id                      INTEGER PRIMARY KEY default nextval('client_id_seq'),
    access_key              VARCHAR(30) NOT NULL,
    name                    TEXT,
    description             TEXT,
    point_of_contact        TEXT,
    resource_owner_key      VARCHAR(30) NOT NULL,
    resource_owner_secret   VARCHAR(30) NOT NULL,
    UNIQUE (access_key)
);
ALTER SEQUENCE client_id_seq owned by client.id;

-- we'll often want to look up a specific client access key, so let's index that:
CREATE INDEX client_access_key_index ON client(access_key);

-- We need to store nonce values in the database. The nonce and timestamp are
-- stored with the access key and the resource owner key.
CREATE TABLE nonce (
    access_key          VARCHAR(30) NOT NULL,
    timestamp           timestamp without time zone NOT NULL,
    nonce               TEXT NOT NULL,
    resource_owner_key  TEXT
);
-- we'll always be searching for entries in all columns, so index that:
CREATE UNIQUE INDEX nonce_all_index ON nonce(access_key, timestamp, nonce, resource_owner_key);
-- We'll also be cleaning this table out periodically by sorting on the timestamp column:
CREATE INDEX nonce_timestamp_index ON nonce(timestamp);
