# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""V1 of the REST API.

This file contains the views for the v1 API, as well as the URL routing table
for the v1 API.

"""
import json
from pyramid.view import view_config

from nfdc import database
from nfdc.auth import oauth_protected


def configure_routes(config):
    """Add url routes to 'config' object."""
    config.add_route('v1.root', '/')
    config.add_route('v1.project', '/{project}')
    config.add_route('v1.test', '/{project}/{test}')
    config.add_route('v1.post', '/{project}/{test}/post')
    # config.scan()


@view_config(route_name='v1.root', renderer='json', request_method='GET')
def root(request):
    """Return summary data about the projects and tests."""
    data = database.get_details_for_all_projects(request.database())
    return dict(
        projects={
            r[0]: {
                'tests': json.loads(r[1]),
                'last_updated': r[2].isoformat(),
                'path': request.route_path('v1.project', project=r[0])
            } for r in data},
    )


@view_config(route_name='v1.project', renderer='json', request_method='GET')
def project(request):
    """Return detailed information about this project's tests."""
    project_name = request.matchdict['project']
    project_data = json.loads(
        database.get_details_for_project(
            request.database(),
            project_name
        )
    )
    for i, test in enumerate(project_data):
        project_data[i]['path'] = request.route_path(
            'v1.test',
            project=project_name,
            test=test['test_name']
        )
    return dict(project_name=project_name, tests=project_data)


@view_config(route_name='v1.test', renderer='json', request_method='GET')
def test(request):
    """Return data points for this particular test.

    Check the Range HTTP header, or return the last 30 days worth of data if
    it's not present.

    """
    project_name = request.matchdict['project']
    test_name = request.matchdict['test']
    data = json.loads(
        database.get_details_for_test(
            request.database(),
            project_name,
            test_name,
        )
    )
    return dict(project_name=project_name, test_name=test_name, data=data)


@view_config(route_name='v1.test', renderer='json', request_method='POST')
def test_add(request):
    """Add data points to this test.

    Return the created id of the data point added to the DB if it was
    successful, or an error if it wasn't.

    TODO: Secure this method somehow. Oauth2 token?
    """
    return dict(created_id=123)


@view_config(route_name='v1.post', renderer='json', request_method='GET')
@oauth_protected()
def test_method(request):
    return dict(status="Success!!!")
