# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Functions that access the database."""

import os.path
import functools
import json
import logging

logger = logging.getLogger(__name__)

import psycopg2.pool


def get_settings_path():
    return os.path.abspath(
        os.path.join(
            os.path.dirname(__file__),
            '..',
            '..',
            'pgsql.json'
        )
    )


def get_db_settings(settings_path):
    """Return a dictionary containing the database settings, as provided by
    juju.

    If the settings file does not exist an IOError is returned.
    If the settings file exists, but is not valid JSON, a ValueError is
        returned.

    """

    with open(settings_path, 'r') as settings_file:
        return json.load(settings_file)


def translate_juju_db_settings(db_settings):
    """Given a dictionary of juju database settings, return a dictionary of
    psycopg2 database connection settings.

    """
    return dict(
        database=db_settings['database'],
        user=db_settings['schema_user'],
        password=db_settings['schema_password'],
        host=db_settings['host'],
        port=db_settings['port'],
    )


@functools.lru_cache()
def get_connection_for_request(request=None):
    """Get a database connection.

    This is bound to every request object we recieve as 'request.database'.

    """
    juju_db_settings = get_db_settings(get_settings_path())
    db_kwargs = translate_juju_db_settings(juju_db_settings)
    return psycopg2.connect(**db_kwargs)


def autocommit(fn):
    """A simple utility decorator that wraps a method that gets content from
    the database. Using this will ensure that:

    1) If the wrapped function returns without raising an exception, the
       connection's 'commit()' method is called.
    2) If the wrapped function raises an exception, the connection's
       'rollback()' method will be called.

    This is important since by default, any operation (including simple
    SELECT queries) starts a new transaction. Without calling 'commit' or
    'rollback' these transactions gradually use up more and more resources.

    """
    def worker(connection, *args, **kwargs):
        try:
            result = fn(connection, *args, **kwargs)
            connection.commit()
            return result
        except Exception:
            connection.rollback()
            raise
    return worker


@autocommit
def get_details_for_all_projects(connection):
    """Get high level details for all projects in the system.

    The return value will be a list of lists. Each inner list will contain:

        [project_name, test_array, max_date]

    * project_name is obvious - it's the name of the project.
    * test_array is a json formatted array of test names that have had data
      entered for this project.
    * max_date is the date of the last data point for this project in any test.

    """
    cursor = connection.cursor()
    cursor.execute('''
        SELECT
            project.name,
            json_agg(distinct test.name),
            max(data.date_entered)
        FROM project
        INNER JOIN test ON project.id = test.project_id
        INNER JOIN data ON test.id = data.test_id
        GROUP BY project.name;
    ''')
    return cursor.fetchall()


@autocommit
def get_details_for_project(connection, project_name):
    """Get high-level details for a single project.

    The return value will be a json string containing the test name, first and
    last data point dates, and a count of data points, in an array, for each
    test.

    """
    cursor = connection.cursor()
    cursor.execute(
        '''
        SELECT
            json_agg(data) as data
        FROM (
            SELECT
                test.name as test_name,
                min(data.date_entered) as first_data,
                max(data.date_entered) as last_data,
                count(data.id) as data_points
            FROM test
            INNER JOIN data on data.test_id = test.id
            INNER JOIN project on test.project_id = project.id
            WHERE project.name = %s
            GROUP BY test.name
        ) as data;
        ''',
        (project_name,),
    )
    return cursor.fetchone()[0]


@autocommit
def get_details_for_test(connection, project_name, test_name):
    cursor = connection.cursor()
    cursor.execute(
        '''
        SELECT json_agg(d)
        FROM (
            SELECT
                data.date_entered AS date_entered,
                data.id AS id,
                data.data AS data
            FROM data
            INNER JOIN test on data.test_id = test.id
            INNER JOIN project on test.project_id = project.id
            WHERE project.name = %s
            AND test.name = %s
        ) as d;
        ''',
        (project_name, test_name)
    )
    logger.info(cursor.query)
    return cursor.fetchone()[0]


@autocommit
def get_auth_client_key_exists(connection, client_key):
    cursor = connection.cursor()
    cursor.execute(
        'SELECT id FROM client WHERE access_key = %s;',
        (client_key,)
    )
    return cursor.rowcount != 0


@autocommit
def get_auth_resource_owner_key_for_client_key(connection, client_key):
    cursor = connection.cursor()
    cursor.execute(
        'SELECT resource_owner_key FROM client WHERE access_key = %s;',
        (client_key,)
    )
    return cursor.fetchone()[0]


@autocommit
def auth_add_client_details(connection, name, description, poc, access_key,
    owner_key, owner_secret):
    cursor = connection.cursor()
    cursor.execute(
        '''
        INSERT INTO client (access_key, name, description, point_of_contact,
            resource_owner_key, resource_owner_secret)
        VALUES (%s, %s, %s, %s, %s, %s);
        ''',
        (access_key, name, description, poc, owner_key, owner_secret)
    )


@autocommit
def get_auth_resource_owner_secret_for_client_key(connection, client_key):
    cursor = connection.cursor()
    cursor.execute(
        'SELECT resource_owner_secret FROM client where access_key = %s;',
        (client_key,)
    )
    return cursor.fetchone()[0]


@autocommit
def get_auth_nonce_already_used(conn, client_key, timestamp, nonce, owner_key):
    cursor = conn.cursor()
    cursor.execute(
        '''
        SELECT * FROM nonce
        WHERE access_key = %s
        AND timestamp = to_timestamp(%s)
        AND nonce = %s
        AND resource_owner_key = %s;
        ''',
        (client_key, timestamp, nonce, owner_key)
    )
    return cursor.rowcount != 0


@autocommit
def store_nonce(connection, client_key, timestamp, nonce, owner_key):
    cursor = connection.cursor()
    cursor.execute(
        '''
        INSERT INTO nonce (access_key, timestamp, nonce, resource_owner_key)
        VALUES (%s, to_timestamp(%s), %s, %s);
        ''',
        (client_key, timestamp, nonce, owner_key)
    )
    return
