import tempfile
import unittest

from mock import patch

from nfdc import db_migrate as dbm


class DBMigrationTests(unittest.TestCase):

    def test_get_settings_returns_dict_on_no_file(self):
        with self.assertRaises(IOError):
            dbm.get_db_settings('/this/path/does/not/exist')

    def test_get_settings_returns_dict_on_bad_json(self):
        with tempfile.NamedTemporaryFile(mode='w') as f:
            f.write('sdfsdf')
            f.flush()

            with self.assertRaises(ValueError):
                    dbm.get_db_settings(f.name)

    def test_get_settings_works_on_example_json(self):
        with tempfile.NamedTemporaryFile(mode='w') as f:
            f.write(
                '{"database": "ci-airline-nfdc-restish", '
                '"allowed-units": "ci-airline-nfdc-restish/0", '
                '"state": "standalone", '
                '"schema_user": "db_2_ci_airline_nfdc_restish_schema", '
                '"schema_password": "secret", '
                '"private-address": "10.0.3.236", '
                '"host": "10.0.3.236", '
                '"user": "db_2_ci_airline_nfdc_restish", '
                '"password": "secret", '
                '"port": "5432"}'
            )
            f.flush()

            expected = {
                "database": "ci-airline-nfdc-restish",
                "allowed-units": "ci-airline-nfdc-restish/0",
                "state": "standalone",
                "schema_user": "db_2_ci_airline_nfdc_restish_schema",
                "schema_password": "secret",
                "private-address": "10.0.3.236",
                "host": "10.0.3.236",
                "user": "db_2_ci_airline_nfdc_restish",
                "password": "secret",
                "port": "5432"
            }
            self.assertEqual(
                expected,
                dbm.get_db_settings(f.name)
            )

    def test_get_maximum_version_works_with_single_patch(self):
        with patch.object(dbm, 'get_all_patch_paths') as p:
            p.return_value = ['000-foo.sql']

            self.assertEqual(0, dbm.get_maximum_version())

    def test_get_maximum_version_works_with_many_patches(self):
        with patch.object(dbm, 'get_all_patch_paths') as p:
            p.return_value = ['000-foo.sql', '001-bar.sql', '002-baz.sql']

            self.assertEqual(2, dbm.get_maximum_version())
